@@ -5,7 +5,8 @@ from pickle import load as pickle_load
 from psycopg2 import connect as psycopg2_connect
 from csv import QUOTE_NONNUMERIC
 from pyspark.sql.session import SparkSession
-from pyspark.sql.dataframe import DataFrame
+from pyspark.sql.dataframe import DataFrame as SparkDF
+from pandas import DataFrame as PandasDF
 from .S3File import S3Files
 
 
@@ -114,7 +115,7 @@ class S3:
 	def write_csv(self, data, path, index=False, encoding='utf-8', **kwargs):
 		"""
 
-		:type data: DataFrame
+		:type data: SparkDF
 		:param path:
 		:param index:
 		:param kwargs:
@@ -135,19 +136,33 @@ class S3:
 			df = read_csv(f, encoding=encoding, **kwargs)
 		return df
 
-	def write_pickle(self, obj, path):
+	def write_pickle(self, obj, path, mode='overwite'):
 		"""
 
-		:type obj: DataFrame
-		:param path:
-		:return:
+		:type obj: SparkDF
+		:type path: str
+		:type mode: str
+		:rtype: bool
 		"""
 		path = self._get_absolute_path(path)
+		if self.exists(path=path):
+			if mode == 'overwrite':
+				self.delete(path=path)
+			else:
+				raise FileExistsError(f'File "{path}" exists on S3!')
+
 		with self.file_system.open(path=path, mode='wb') as f:
 			try:
-				obj.to_pickle(f)
+				return obj.to_pickle(f)
 			except:
-				pickle_dump(obj=obj, file=f)
+				if self.exists(path=path):
+					self.delete(path=path)
+				try:
+					return pickle_dump(obj=obj, file=f)
+				except Exception as e:
+					if self.exists(path=path):
+						self.delete(path=path)
+					raise e
 
 	def read_pickle(self, path):
 		path = self._get_absolute_path(path)
@@ -185,10 +200,10 @@ class S3:
 
 		connection.close()
 
-	def save_parquet(self, data, path, mode='overwrite'):
+	def write_parquet(self, data, path, mode='overwrite'):
 		"""
 		saves a Spark DataFrame to a path on S3 and returns the list of parquet files
-		:type data: DataFrame
+		:type data: SparkDF
 		:type path: str
 		:type mode: str
 		:rtype: list[str]
@@ -263,13 +278,13 @@ class S3:
 			n_and_e = self.get_file_name_and_extension(path=path).lower()
 			return n_and_e.startswith('part-') and n_and_e.endswith('.parquet')
 
-	def load_parquet(self, path, spark=None, parallel=True):
+	def read_parquet(self, path, spark=None, parallel=True):
 		"""
 		reads parquet files inside a path and returns the data
 		:type path: str
 		:type spark: SparkSession or NoneType
 		:type parallel: bool
-		:rtype: DataFrame
+		:rtype: SparkDF
 		"""
 		if spark is None:
 			spark = self.spark
@@ -298,3 +313,51 @@ class S3:
 					result = result.union(data)
 
 			return result
+
+	load_parquet = read_parquet
+	save_parquet = write_parquet
+	load_pickle = read_pickle
+	save_pickle = write_pickle
+
+	def write(self, obj, path, mode='overwrite'):
+		"""
+		writes an object to path as a parquet if the object is a Spark DataFrame or a pickle otherwise
+		:type obj: object
+		:type path: str
+		:type mode: str
+		:rtype: bool
+		"""
+		if isinstance(obj, SparkDF):
+			if path.endswith('.pickle'):
+				raise ValueError(f'Spark DataFrame cannot be saved into a pickle file: "{path}"')
+
+			if not path.endswith('.parquet'):
+				path += '.parquet'
+
+			return self.write_parquet(data=obj, path=path, mode=mode)
+
+		else:
+			if path.endswith('.parquet'):
+				raise ValueError(f'{type(path)} cannot be saved into a parquet file: "{path}"')
+
+			if not path.endswith('.pickle'):
+				path += '.pickle'
+
+			return self.write_pickle(obj=obj, path=path, mode=mode)
+
+	def read(self, path, spark=None):
+		"""
+		reads an object from a path. If the extension is .parquet then the object is assumed to be a Spark DataFrame
+		and the read_parquet method is used, otherwise a pickle will be read.
+		:type path: str
+		:type spark: SparkSession
+		:rtype: bool
+		"""
+		if path.endswith('.parquet'):
+			return self.read_parquet(path=path, spark=spark or self.spark)
+
+		elif path.endswith('.pickle'):
+			return self.read_pickle(path=path)
+
+		else:
+			raise NotImplementedError(f'read is not implemented for {path}!')
